# SingSurf
A mathematical curve and surface visualiser for singular surface and objects from Singularity Theory.

The program can calculate many of the objects found in Singularity theory:

* Algebraic curves defined by a single polynomial equation in two variables. e.g. a circle 

            x^2 + y^2 - r^2;
* Algebraic surfaces defined by a single polynomial equation in three variables. e.g. a cone

            x^2 + y^2 - z^2;
* Parameterised curves defined by a 3D vector expression in a single variable. e.g. a helix

           [cos(pi t), sin(pi t), t];   
* Parameterised surfaces defined by a 3D vector expression in two variables. e.g. a cross-cap

           [x,x y,y^2];          
* Intersection of surfaces with sets defined by an equation. Can be used to calculate non-polynomial curves.
* Mapping from R^3 to R^3 defined by 3D vector equation in three variables. e.g. a rotation

		   [cos(pi th) x - sin(pi th) y,sin(pi th) x + cos(pi th) y,z];   	   
* Intersections where the equation depends on the definition of another curve or surface. e.g. The profile of a surface

			N . [A,B,C];
			N = diff(S,x) ^^ diff(S,y);	
* Mappings where the equation depends on another surface. For example projection of a curve onto a surface.
* Intersections where the equations depends on a pair of curves. For example the pre-symmetry set of a curve.
* Mapping where the equation depends on a pair of curves. For example the Symmetry set.

## Requirements

The program requires

* Java version 8 (later versions of Java will not work) you can get from
[oracle](https://www.oracle.com/technetwork/java/javase/downloads/jre8-downloads-2133155.html)
* JavaView mathematical visualization software from [javaview.de](http://www.javaview.de/) Alas not open-source
* JEP 2.4.1 Java Expression Parser, a mathematical parser evaluator from [jep-java-gpl](https://github.com/nathanfunk/jep-java-gpl) or my own fork [jep-java-gpl](https://github.com/RichardMorris/jep-java-gpl) which may contain some customisation of the package to fit the needs of the SingSurf program.

It is a good idea to register your version of JavaView. Regestration provides a licence file `jv-lic.lic` which should be copied to the `rsrc` directory, this prevents a notification message appearing.    

## Installation and running

The zip file contains all the necessary files for execution.

There are three different main classes
* `org.singsurf.singsurf.SingSurfPro` the 3D version with all sub-types

* `org.singsurf.singsurf.SingSurf2D` the 2D version optimised for curves in the plane

* `org.singsurf.singsurf.ASurfSimp` a simplified version just with the algebraic surface component

To run the 3D version use

        java -jar SingSurf.jar    
To run the 2D version use

        java -cp SingSurf.jar org.singsurf.singsurf.SingSurf2D
        
## User interface

Once the program starts you should see two windows. 

![user interface](doc/ui2.png) 

The left hand window holds the various control panels, used to set the definitions of surfaces etc.
The right hand window is the main display showing the calculated geometries.

The menus in the control window are all JavaView options. These allow actions like saving the generated geometry models, 
controlling detailed appearance properties and other JavaView options. Details on these can be found
at [www.javaview.de/doc](http://www.javaview.de/doc/index.html).

The display window menus relate to the SingSurf program. Here you can load a definition, 
or use one of the predefined examples.

The list of the left hand side of the display window shows the different surfaces and operators. 
Clicking on the name will select the item. When selected the controls for 
this item are displayed in control window.

These controls have four tabs: definition, domain, parameters and options. 
In the main definition tab the main definition can be input. The format of this depends of the
type of surface/operator.

#### Definition panel

Contains the expression defining the surface

![Definition panel](doc/def_pan.png)

#### Domain panel 

Controls the range of the input variables, number of steps to take.

![Domain panel](doc/dom_pan.png)

#### Parameter panel

Control any parameters which appears in the definition.

![Parameter panel](doc/param_pan.png)

#### Options panel

Various options controlling the calculation and display of the surface. 

![Options panel](doc/opt_pan.png)

#### Javaview panels

Clicking on the `Display` item at the top of the display window list will bring up the 
JavaView display control.  Here various features of rendering can be controlled. The `Inspector` menu brings up  
further JavaView control panels controlling the Camera, Lights and properties of the Geometry. 

## Types of curve, surface and operators

Basic curves and surfaces:

* __Algebraic curves__: one expression in two variables, either `x^2 + y^2 - r^2`; or with an equality `x^2 + y^2 = r^2`;
* __Algebraic surfaces__: one expression in three variables `x^2+y^2+z^2 = r^2`;
* __Parameterised curve__: a 3D vector expression in one variable `[x,x^2,x^3]`;
* __Parameterised surface__: a 3D vector expression in two variables `[x,y,A x^2+2 B x y + C y^2]`;

Basic operators:

These read in a geometry, apply the operation and produce another geometry as output. Each can have one or more geometries as input.

* __Intersection__: one expression in three variables `x^2+y^2+z^2 = r^2`;
* __Clip__: one expression in three variables `x^2+y^2+z^2 = r^2`;
* __Mapping__: a 3D vector expression in three variables `[y,-x,z]`;

Advanced operators:

Here the definition of the operator depends on the definition of a curve or surface,
called the ingredient. Equation depending on three variables `x`,`y`,`z` and one or two 
ingredients, `S`, `T` representing maps S:(x,y,z)->**R**^3 or S:(x,y,z)->**R**. Expressions can use the variable `S` say `S . [a,b,c];` or it derivatives `diff(S,x)`. For example a profile of a parameterised
surface could be specified as

```
N . [a,b,c];
N = [diff(S,x),diff(S,y),diff(S,z)];
```

Four different advance operators are possible

* __GenIntersection__: intersection depending on one ingredient `S`. Examples include profiles of surfaces, parabolic lines
* __GenMapping__: mapping depends on one ingredient `S`. Examples include projecting curves in parameter space onto a surface, the Guass map and focal surface.
* __BiInt__: intersection depending on two ingredients `S`, `T`, these can often be the same curve. Example includes the pre-Symmetry Sets.
* __BiMap__: mapping depends on two ingredients `S`, `T`, used to generate the symmetry set from the pre-symmetry set. 



   


					               
